@@ -199,9 +199,9 @@ typedef
       /* Emulation notes */
       UInt   guest_EMNOTE;
 
-      /* For clflush: record start and length of area to invalidate */
-      UInt guest_TISTART;
-      UInt guest_TILEN;
+      /* For clflush/clinval: record start and length of area */
+      UInt guest_CMSTART;
+      UInt guest_CMLEN;
 
       /* Used to record the unredirected guest address at the start of
          a translation whose start has been redirected.  By reading
@@ -220,8 +220,8 @@ typedef
          been interrupted by a signal. */
       UInt guest_IP_AT_SYSCALL;
 
-      /* Padding to make it have an 32-aligned size */
-      UInt padding[5];
+      /* Padding to make it have an 16-aligned size */
+      UInt padding1;
    }
    VexGuestX86State;
 
