@@ -27,7 +27,7 @@
 #include "regcache.h"
 
 #include "pub_core_aspacemgr.h"
-#include "pub_tool_machine.h"
+#include "pub_core_machine.h"
 #include "pub_core_threadstate.h"
 #include "pub_core_transtab.h"
 #include "pub_core_gdbserver.h" 
@@ -323,7 +323,7 @@ void transfer_register (ThreadId tid, int abs_regno, void * buf,
 }
 
 static
-char* target_xml (Bool shadow_mode)
+const char* target_xml (Bool shadow_mode)
 {
    if (shadow_mode) {
       return "powerpc-altivec32l-valgrind.xml";
@@ -332,6 +332,15 @@ char* target_xml (Bool shadow_mode)
    }  
 }
 
+static CORE_ADDR** target_get_dtv (ThreadState *tst)
+{
+   VexGuestPPC32State* ppc32 = (VexGuestPPC32State*)&tst->arch.vex;
+   // ppc32 dtv is located just before the tcb, which is 0x7000 before
+   // the thread id (r2)
+   return (CORE_ADDR**)((CORE_ADDR)ppc32->guest_GPR2
+                        - 0x7000 - sizeof(CORE_ADDR));
+}
+
 static struct valgrind_target_ops low_target = {
    num_regs,
    regs,
@@ -340,7 +349,8 @@ static struct valgrind_target_ops low_target = {
    get_pc,
    set_pc,
    "ppc32",
-   target_xml
+   target_xml,
+   target_get_dtv
 };
 
 void ppc32_init_architecture (struct valgrind_target_ops *target)
