@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2011 Julian Seward 
+   Copyright (C) 2000-2013 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -31,14 +31,14 @@
 #include "pub_core_basics.h"
 #include "pub_core_vki.h"
 #include "pub_core_debuglog.h"
-#include "pub_core_gdbserver.h"
+#include "pub_core_gdbserver.h"  // VG_(gdb_printf)
 #include "pub_core_libcbase.h"
 #include "pub_core_libcassert.h"
 #include "pub_core_libcfile.h"   // VG_(write)(), VG_(write_socket)()
 #include "pub_core_libcprint.h"
 #include "pub_core_libcproc.h"   // VG_(getpid)(), VG_(read_millisecond_timer()
 #include "pub_core_options.h"
-#include "valgrind.h"            // For RUNNING_ON_VALGRIND
+#include "pub_core_clreq.h"      // For RUNNING_ON_VALGRIND
 
 
 /* ---------------------------------------------------------------------
@@ -57,7 +57,7 @@ OutputSink VG_(xml_output_sink) = { -1, False }; /* disabled */
  
 /* Do the low-level send of a message to the logging sink. */
 static
-void send_bytes_to_logging_sink ( OutputSink* sink, Char* msg, Int nbytes )
+void send_bytes_to_logging_sink ( OutputSink* sink, const HChar* msg, Int nbytes )
 {
    if (sink->is_socket) {
       Int rc = VG_(write_socket)( sink->fd, msg, nbytes );
@@ -74,7 +74,9 @@ void send_bytes_to_logging_sink ( OutputSink* sink, Char* msg, Int nbytes )
          any more output. */
       if (sink->fd >= 0)
          VG_(write)( sink->fd, msg, nbytes );
-      else if (sink->fd == -2)
+      else if (sink->fd == -2 && nbytes > 0)
+         /* send to gdb the provided data, which must be
+            a null terminated string with len >= 1 */
          VG_(gdb_printf)("%s", msg);
    }
 }
@@ -166,6 +168,25 @@ UInt VG_(printf_xml) ( const HChar *format, ... )
    return ret;
 }
 
+static UInt emit_WRK ( const HChar* format, va_list vargs )
+{
+   if (VG_(clo_xml)) {
+      return VG_(vprintf_xml)(format, vargs);
+   } else if (VG_(log_output_sink).fd == -2) {
+      return VG_(vprintf) (format, vargs);
+   } else {
+      return VG_(vmessage)(Vg_UserMsg, format, vargs);
+   }
+}
+UInt VG_(emit) ( const HChar* format, ... )
+{
+   UInt ret;
+   va_list vargs;
+   va_start(vargs, format);
+   ret = emit_WRK(format, vargs);
+   va_end(vargs);
+   return ret;
+}
 
 /* --------- sprintf --------- */
 
@@ -179,7 +200,7 @@ static void add_to__sprintf_buf ( HChar c, void *p )
    *(*b)++ = c;
 }
 
-UInt VG_(vsprintf) ( Char* buf, const HChar *format, va_list vargs )
+UInt VG_(vsprintf) ( HChar* buf, const HChar *format, va_list vargs )
 {
    Int ret;
    HChar* sprintf_ptr = buf;
@@ -193,7 +214,7 @@ UInt VG_(vsprintf) ( Char* buf, const HChar *format, va_list vargs )
    return ret;
 }
 
-UInt VG_(sprintf) ( Char* buf, const HChar *format, ... )
+UInt VG_(sprintf) ( HChar* buf, const HChar *format, ... )
 {
    UInt ret;
    va_list vargs;
@@ -226,20 +247,21 @@ static void add_to__snprintf_buf ( HChar c, void* p )
    } 
 }
 
-UInt VG_(vsnprintf) ( Char* buf, Int size, const HChar *format, va_list vargs )
+UInt VG_(vsnprintf) ( HChar* buf, Int size, const HChar *format, va_list vargs )
 {
    snprintf_buf_t b;
    b.buf      = buf;
    b.buf_size = size < 0 ? 0 : size;
    b.buf_used = 0;
-
+   if (b.buf_size > 0)
+      b.buf[0] = 0; // ensure to null terminate buf if empty format
    (void) VG_(debugLog_vprintf) 
              ( add_to__snprintf_buf, &b, format, vargs );
 
    return b.buf_used;
 }
 
-UInt VG_(snprintf) ( Char* buf, Int size, const HChar *format, ... )
+UInt VG_(snprintf) ( HChar* buf, Int size, const HChar *format, ... )
 {
    UInt ret;
    va_list vargs;
@@ -267,11 +289,11 @@ void VG_(vcbprintf)( void(*char_sink)(HChar, void* opaque),
 
 // Percentify n/m with d decimal places.  Includes the '%' symbol at the end.
 // Right justifies in 'buf'.
-void VG_(percentify)(ULong n, ULong m, UInt d, Int n_buf, char buf[]) 
+void VG_(percentify)(ULong n, ULong m, UInt d, Int n_buf, HChar buf[]) 
 {
    Int i, len, space;
    ULong p1;
-   Char fmt[32];
+   HChar fmt[32];
 
    if (m == 0) {
       // Have to generate the format string in order to be flexible about
@@ -396,11 +418,17 @@ static void add_to__vmessage_buf ( HChar c, void *p )
 
       // Print one '>' in front of the messages for each level of
       // self-hosting being performed.
+      // Do not print such '>' if sim hint "no-inner-prefix" given
+      // (useful to run regression tests in an outer/inner setup
+      // and avoid the diff failing due to these unexpected '>').
       depth = RUNNING_ON_VALGRIND;
-      if (depth > 10)
-         depth = 10; // ?!?!
-      for (i = 0; i < depth; i++) {
-         b->buf[b->buf_used++] = '>';
+      if (depth > 0 
+          && !SimHintiS(SimHint_no_inner_prefix, VG_(clo_sim_hints))) {
+         if (depth > 10)
+            depth = 10; // ?!?!
+         for (i = 0; i < depth; i++) {
+            b->buf[b->buf_used++] = '>';
+         }
       }
 
       if (Vg_FailMsg == b->kind) {
@@ -517,7 +545,7 @@ UInt VG_(fmsg) ( const HChar* format, ... )
    return count;
 }
 
-void VG_(fmsg_bad_option) ( HChar* opt, const HChar* format, ... )
+void VG_(fmsg_bad_option) ( const HChar* opt, const HChar* format, ... )
 {
    va_list vargs;
    va_start(vargs,format);
@@ -568,7 +596,7 @@ void VG_(err_missing_prog) ( void  )
 }
 
 __attribute__((noreturn))
-void VG_(err_config_error) ( Char* format, ... )
+void VG_(err_config_error) ( const HChar* format, ... )
 {
    va_list vargs;
    va_start(vargs,format);
