@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2012 Julian Seward
+   Copyright (C) 2000-2013 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -42,8 +42,9 @@
 
 // Address space globals
 
-extern Addr  VG_(clstk_base);	 // client stack range
-extern Addr  VG_(clstk_end);
+// client stack range
+extern Addr  VG_(clstk_start_base); // *Initial* lowest byte address
+extern Addr  VG_(clstk_end);        // Highest byte address
 extern UWord VG_(clstk_id);      // client stack id
 
 /* linux only: where is the client auxv ? */
@@ -67,6 +68,9 @@ extern Int VG_(cl_exec_fd);
    the file contents alive exactly until the process exits. */
 extern Int VG_(cl_cmdline_fd);
 
+/* Same as above, but for /proc/<pid>/auxv. */
+extern Int VG_(cl_auxv_fd);
+
 // Client's original rlimit data and rlimit stack
 extern struct vki_rlimit VG_(client_rlimit_data);
 extern struct vki_rlimit VG_(client_rlimit_stack);
@@ -90,6 +94,24 @@ extern Addr VG_(client___libc_freeres_wrapper);
 extern Addr VG_(client__dl_sysinfo_int80);
 
 
+/* glibc nptl pthread systems only, when no-nptl-pthread-stackcache
+   was given in --sim-hints.
+   Used for a (kludgy) way to disable the cache of stacks as implemented in
+   nptl glibc. 
+   Based on internal knowledge of the pthread glibc nptl/allocatestack.c code:
+   a huge value in stack_cache_actsize (bigger than the constant
+   stack_cache_maxsize) makes glibc believes the cache is full
+   and so stacks are always released when a pthread terminates.
+   Several ugliness in this kludge:
+    * hardcodes private glibc var name "stack_cache_maxsize"
+    * based on knowledge of the code of the functions
+      queue_stack and __free_stacks
+    * static symbol for "stack_cache_maxsize" must be in
+      the debug info.
+   It would be much cleaner to have a documented and supported
+   way to disable the pthread stack cache. */
+extern SizeT* VG_(client__stack_cache_actsize__addr);
+
 #endif   // __PUB_CORE_CLIENTSTATE_H
 
 /*--------------------------------------------------------------------*/
