@@ -8,7 +8,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2012 Julian Seward
+   Copyright (C) 2000-2013 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -44,7 +44,8 @@
 #define VG_DEEPEST_BACKTRACE 500
 
 // Print stats (informational only).
-extern void VG_(print_ExeContext_stats) ( void );
+// If with_stacktraces, outputs all the recorded stacktraces.
+extern void VG_(print_ExeContext_stats) ( Bool with_stacktraces );
 
 // Extract the StackTrace from an ExeContext.
 // (Minor hack: we use Addr* as the return type instead of StackTrace so
