@@ -8,7 +8,7 @@
    This file is part of Helgrind, a Valgrind tool for detecting errors
    in threaded programs.
 
-   Copyright (C) 2007-2012 OpenWorks Ltd
+   Copyright (C) 2007-2013 OpenWorks Ltd
       info@open-works.co.uk
 
    This program is free software; you can redistribute it and/or
@@ -40,8 +40,10 @@
 #include "pub_tool_debuginfo.h"
 #include "pub_tool_threadstate.h"
 #include "pub_tool_options.h"     // VG_(clo_xml)
+#include "pub_tool_addrinfo.h"
 
 #include "hg_basics.h"
+#include "hg_addrdescr.h"
 #include "hg_wordset.h"
 #include "hg_lock_n_thread.h"
 #include "libhb.h"
@@ -66,7 +68,7 @@ static Word string_table_cmp ( UWord s1, UWord s2 ) {
    return (Word)VG_(strcmp)( (HChar*)s1, (HChar*)s2 );
 }
 
-static HChar* string_table_strdup ( HChar* str ) {
+static HChar* string_table_strdup ( const HChar* str ) {
    HChar* copy = NULL;
    HG_(stats__string_table_queries)++;
    if (!str)
@@ -77,14 +79,14 @@ static HChar* string_table_strdup ( HChar* str ) {
       tl_assert(string_table);
    }
    if (VG_(lookupFM)( string_table,
-                      NULL, (Word*)&copy, (Word)str )) {
+                      NULL, (UWord*)&copy, (UWord)str )) {
       tl_assert(copy);
       if (0) VG_(printf)("string_table_strdup: %p -> %p\n", str, copy );
       return copy;
    } else {
       copy = HG_(strdup)("hg.sts.2", str);
       tl_assert(copy);
-      VG_(addToFM)( string_table, (Word)copy, (Word)copy );
+      VG_(addToFM)( string_table, (UWord)copy, (UWord)copy );
       return copy;
    }
 }
@@ -165,7 +167,7 @@ static Lock* mk_LockP_from_LockN ( Lock* lkn,
                                    HG_(free), lock_unique_cmp );
       tl_assert(map_LockN_to_P);
    }
-   if (!VG_(lookupFM)( map_LockN_to_P, NULL, (Word*)&lkp, (Word)lkn)) {
+   if (!VG_(lookupFM)( map_LockN_to_P, NULL, (UWord*)&lkp, (UWord)lkn)) {
       lkp = HG_(zalloc)( "hg.mLPfLN.2", sizeof(Lock) );
       *lkp = *lkn;
       lkp->admin_next = NULL;
@@ -178,7 +180,7 @@ static Lock* mk_LockP_from_LockN ( Lock* lkn,
       lkp->heldBy = NULL;
       lkp->acquired_at = NULL;
       lkp->hbso = NULL;
-      VG_(addToFM)( map_LockN_to_P, (Word)lkp, (Word)lkp );
+      VG_(addToFM)( map_LockN_to_P, (UWord)lkp, (UWord)lkp );
    }
    tl_assert( HG_(is_sane_LockP)(lkp) );
    return lkp;
@@ -291,16 +293,10 @@ typedef
          struct {
             Addr        data_addr;
             Int         szB;
+            AddrInfo    data_addrinfo;
             Bool        isWrite;
             Thread*     thr;
             Lock**      locksHeldW;
-            /* descr1/2 provide a description of stack/global locs */
-            XArray*     descr1; /* XArray* of HChar */
-            XArray*     descr2; /* XArray* of HChar */
-            /* halloc/haddr/hszB describe the addr if it is a heap block. */
-            ExeContext* hctxt;
-            Addr        haddr;
-            SizeT       hszB;
             /* h1_* and h2_* provide some description of a previously
                observed access with which we are conflicting. */
             Thread*     h1_ct; /* non-NULL means h1 info present */
@@ -335,8 +331,8 @@ typedef
             Thread*     thr;
             /* The first 4 fields describe the previously observed
                (should-be) ordering. */
-            Addr        shouldbe_earlier_ga;
-            Addr        shouldbe_later_ga;
+            Lock*       shouldbe_earlier_lk;
+            Lock*       shouldbe_later_lk;
             ExeContext* shouldbe_earlier_ec;
             ExeContext* shouldbe_later_ec;
             /* In principle we need to record two more stacks, from
@@ -411,52 +407,7 @@ UInt HG_(update_extra) ( Error* err )
          VG_(printf)("HG_(update_extra): "
                      "%d conflicting-event queries\n", xxx);
 
-      tl_assert(!xe->XE.Race.hctxt);
-      tl_assert(!xe->XE.Race.descr1);
-      tl_assert(!xe->XE.Race.descr2);
-
-      /* First, see if it's in any heap block.  Unfortunately this
-         means a linear search through all allocated heap blocks.  The
-         assertion says that if it's detected as a heap block, then we
-         must have an allocation context for it, since all heap blocks
-         should have an allocation context. */
-      Bool is_heapblock
-         = HG_(mm_find_containing_block)( 
-              &xe->XE.Race.hctxt, &xe->XE.Race.haddr, &xe->XE.Race.hszB,
-              xe->XE.Race.data_addr
-           );
-      tl_assert(is_heapblock == (xe->XE.Race.hctxt != NULL));
-
-      if (!xe->XE.Race.hctxt) {
-         /* It's not in any heap block.  See if we can map it to a
-            stack or global symbol. */
-
-         xe->XE.Race.descr1
-            = VG_(newXA)( HG_(zalloc), "hg.update_extra.Race.descr1",
-                          HG_(free), sizeof(HChar) );
-         xe->XE.Race.descr2
-            = VG_(newXA)( HG_(zalloc), "hg.update_extra.Race.descr2",
-                          HG_(free), sizeof(HChar) );
-
-         (void) VG_(get_data_description)( xe->XE.Race.descr1,
-                                           xe->XE.Race.descr2,
-                                           xe->XE.Race.data_addr );
-
-         /* If there's nothing in descr1/2, free it.  Why is it safe to
-            to VG_(indexXA) at zero here?  Because
-            VG_(get_data_description) guarantees to zero terminate
-            descr1/2 regardless of the outcome of the call.  So there's
-            always at least one element in each XA after the call.
-         */
-         if (0 == VG_(strlen)( VG_(indexXA)( xe->XE.Race.descr1, 0 ))) {
-            VG_(deleteXA)( xe->XE.Race.descr1 );
-            xe->XE.Race.descr1 = NULL;
-         }
-         if (0 == VG_(strlen)( VG_(indexXA)( xe->XE.Race.descr2, 0 ))) {
-            VG_(deleteXA)( xe->XE.Race.descr2 );
-            xe->XE.Race.descr2 = NULL;
-         }
-      }
+      HG_(describe_addr) (xe->XE.Race.data_addr, &xe->XE.Race.data_addrinfo);
 
       /* And poke around in the conflicting-event map, to see if we
          can rustle up a plausible-looking conflicting memory access
@@ -524,6 +475,8 @@ void HG_(record_error_Race) ( Thread* thr,
      if (sect == Vg_SectGOTPLT) return;
      /* SectPLT is required on ppc32/64-linux */
      if (sect == Vg_SectPLT) return;
+     /* SectGOT is required on arm-linux */
+     if (sect == Vg_SectGOT) return;
    }
 #  endif
 
@@ -538,8 +491,7 @@ void HG_(record_error_Race) ( Thread* thr,
    /* Skip on the detailed description of the raced-on address at this
       point; it's expensive.  Leave it for the update_extra function
       if we ever make it that far. */
-   tl_assert(xe.XE.Race.descr1 == NULL);
-   tl_assert(xe.XE.Race.descr2 == NULL);
+   xe.XE.Race.data_addrinfo.tag = Addr_Undescribed;
    // FIXME: tid vs thr
    // Skip on any of the conflicting-access info at this point.
    // It's expensive to obtain, and this error is more likely than
@@ -616,8 +568,8 @@ void HG_(record_error_UnlockBogus) ( Thread* thr, Addr lock_ga )
 
 void HG_(record_error_LockOrder)(
         Thread*     thr, 
-        Addr        shouldbe_earlier_ga,
-        Addr        shouldbe_later_ga,
+        Lock*       shouldbe_earlier_lk,
+        Lock*       shouldbe_later_lk,
         ExeContext* shouldbe_earlier_ec,
         ExeContext* shouldbe_later_ec,
         ExeContext* actual_earlier_ec
@@ -629,9 +581,13 @@ void HG_(record_error_LockOrder)(
    init_XError(&xe);
    xe.tag = XE_LockOrder;
    xe.XE.LockOrder.thr       = thr;
-   xe.XE.LockOrder.shouldbe_earlier_ga = shouldbe_earlier_ga;
+   xe.XE.LockOrder.shouldbe_earlier_lk 
+      = mk_LockP_from_LockN(shouldbe_earlier_lk, 
+                            False/*!allowed_to_be_invalid*/);
    xe.XE.LockOrder.shouldbe_earlier_ec = shouldbe_earlier_ec;
-   xe.XE.LockOrder.shouldbe_later_ga   = shouldbe_later_ga;
+   xe.XE.LockOrder.shouldbe_later_lk   
+      = mk_LockP_from_LockN(shouldbe_later_lk, 
+                            False/*!allowed_to_be_invalid*/);
    xe.XE.LockOrder.shouldbe_later_ec   = shouldbe_later_ec;
    xe.XE.LockOrder.actual_earlier_ec   = actual_earlier_ec;
    // FIXME: tid vs thr
@@ -641,8 +597,8 @@ void HG_(record_error_LockOrder)(
                             XE_LockOrder, 0, NULL, &xe );
 }
 
-void HG_(record_error_PthAPIerror) ( Thread* thr, HChar* fnname, 
-                                     Word err, HChar* errstr )
+void HG_(record_error_PthAPIerror) ( Thread* thr, const HChar* fnname, 
+                                     Word err, const HChar* errstr )
 {
    XError xe;
    tl_assert( HG_(is_sane_Thread)(thr) );
@@ -661,8 +617,8 @@ void HG_(record_error_PthAPIerror) ( Thread* thr, HChar* fnname,
                             XE_PthAPIerror, 0, NULL, &xe );
 }
 
-void HG_(record_error_Misc_w_aux) ( Thread* thr, HChar* errstr,
-                                    HChar* auxstr, ExeContext* auxctx )
+void HG_(record_error_Misc_w_aux) ( Thread* thr, const HChar* errstr,
+                                    const HChar* auxstr, ExeContext* auxctx )
 {
    XError xe;
    tl_assert( HG_(is_sane_Thread)(thr) );
@@ -680,7 +636,7 @@ void HG_(record_error_Misc_w_aux) ( Thread* thr, HChar* errstr,
                             XE_Misc, 0, NULL, &xe );
 }
 
-void HG_(record_error_Misc) ( Thread* thr, HChar* errstr )
+void HG_(record_error_Misc) ( Thread* thr, const HChar* errstr )
 {
    HG_(record_error_Misc_w_aux)(thr, errstr, NULL, NULL);
 }
@@ -739,7 +695,7 @@ Bool HG_(eq_Error) ( VgRes not_used, Error* e1, Error* e2 )
 /* Do a printf-style operation on either the XML or normal output
    channel, depending on the setting of VG_(clo_xml).
 */
-static void emit_WRK ( HChar* format, va_list vargs )
+static void emit_WRK ( const HChar* format, va_list vargs )
 {
    if (VG_(clo_xml)) {
       VG_(vprintf_xml)(format, vargs);
@@ -747,8 +703,8 @@ static void emit_WRK ( HChar* format, va_list vargs )
       VG_(vmessage)(Vg_UserMsg, format, vargs);
    }
 }
-static void emit ( HChar* format, ... ) PRINTF_CHECK(1, 2);
-static void emit ( HChar* format, ... )
+static void emit ( const HChar* format, ... ) PRINTF_CHECK(1, 2);
+static void emit ( const HChar* format, ... )
 {
    va_list vargs;
    va_start(vargs, format);
@@ -807,7 +763,6 @@ static Bool announce_one_thread ( Thread* thr )
    return True;
 }
 
-
 /* Announce 'lk'. */
 static void announce_LockP ( Lock* lk )
 {
@@ -815,15 +770,24 @@ static void announce_LockP ( Lock* lk )
    if (lk == Lock_INVALID)
       return; /* Can't be announced -- we know nothing about it. */
    tl_assert(lk->magic == LockP_MAGIC);
-   if (!lk->appeared_at)
-     return; /* There's nothing we can show */
 
    if (VG_(clo_xml)) {
-      /* fixme: add announcement */
+      if (lk->appeared_at) {
+         emit( "  <auxwhat>Lock at %p was first observed</auxwhat>\n",
+               (void*)lk );
+         VG_(pp_ExeContext)( lk->appeared_at );
+      }
+
    } else {
-      VG_(umsg)( "Lock at %p was first observed\n",
-                 (void*)lk->guestaddr );
-      VG_(pp_ExeContext)( lk->appeared_at );
+      if (lk->appeared_at) {
+         VG_(umsg)( " Lock at %p was first observed\n",
+                    (void*)lk->guestaddr );
+         VG_(pp_ExeContext)( lk->appeared_at );
+      } else {
+         VG_(umsg)( " Lock at %p : no stacktrace for first observation\n",
+                    (void*)lk->guestaddr );
+      }
+      HG_(get_and_pp_addrdescr) (lk->guestaddr);
       VG_(umsg)("\n");
    }
 }
@@ -848,7 +812,7 @@ static void announce_combined_LockP_vecs ( Lock** lockvec,
 }
 
 
-static void show_LockP_summary_textmode ( Lock** locks, HChar* pre )
+static void show_LockP_summary_textmode ( Lock** locks, const HChar* pre )
 {
    tl_assert(locks);
    UWord i;
@@ -916,6 +880,17 @@ void HG_(before_pp_Error) ( Error* err )
             announce_one_thread( xe->XE.Race.h2_ct );
          if (xe->XE.Race.h1_ct)
             announce_one_thread( xe->XE.Race.h1_ct );
+         if (xe->XE.Race.data_addrinfo.Addr.Block.alloc_tinfo.tnr) {
+            Thread* thr = get_admin_threads();
+            while (thr) {
+               if (thr->errmsg_index 
+                   == xe->XE.Race.data_addrinfo.Addr.Block.alloc_tinfo.tnr) {
+                  announce_one_thread (thr);
+                  break;
+               }
+               thr = thr->admin;
+            }
+         }
          break;
       default:
          tl_assert(0);
@@ -984,8 +959,8 @@ void HG_(pp_Error) ( Error* err )
          emit( "    <text>Thread #%d: lock order \"%p before %p\" "
                     "violated</text>\n",
                (Int)xe->XE.LockOrder.thr->errmsg_index,
-               (void*)xe->XE.LockOrder.shouldbe_earlier_ga,
-               (void*)xe->XE.LockOrder.shouldbe_later_ga );
+               (void*)xe->XE.LockOrder.shouldbe_earlier_lk->guestaddr,
+               (void*)xe->XE.LockOrder.shouldbe_later_lk->guestaddr );
          emit( "    <hthreadid>%d</hthreadid>\n",
                (Int)xe->XE.LockOrder.thr->errmsg_index );
          emit( "  </xwhat>\n" );
@@ -994,24 +969,26 @@ void HG_(pp_Error) ( Error* err )
              && xe->XE.LockOrder.shouldbe_later_ec) {
             emit( "  <auxwhat>Required order was established by "
                   "acquisition of lock at %p</auxwhat>\n",
-                  (void*)xe->XE.LockOrder.shouldbe_earlier_ga );
+                  (void*)xe->XE.LockOrder.shouldbe_earlier_lk->guestaddr );
             VG_(pp_ExeContext)( xe->XE.LockOrder.shouldbe_earlier_ec );
             emit( "  <auxwhat>followed by a later acquisition "
                   "of lock at %p</auxwhat>\n",
-                  (void*)xe->XE.LockOrder.shouldbe_later_ga );
+                  (void*)xe->XE.LockOrder.shouldbe_later_lk->guestaddr );
             VG_(pp_ExeContext)( xe->XE.LockOrder.shouldbe_later_ec );
          }
+         announce_LockP ( xe->XE.LockOrder.shouldbe_earlier_lk );
+         announce_LockP ( xe->XE.LockOrder.shouldbe_later_lk );
 
       } else {
 
          emit( "Thread #%d: lock order \"%p before %p\" violated\n",
                (Int)xe->XE.LockOrder.thr->errmsg_index,
-               (void*)xe->XE.LockOrder.shouldbe_earlier_ga,
-               (void*)xe->XE.LockOrder.shouldbe_later_ga );
+               (void*)xe->XE.LockOrder.shouldbe_earlier_lk->guestaddr,
+               (void*)xe->XE.LockOrder.shouldbe_later_lk->guestaddr );
          emit( "\n" );
          emit( "Observed (incorrect) order is: "
                "acquisition of lock at %p\n",
-               (void*)xe->XE.LockOrder.shouldbe_later_ga);
+               (void*)xe->XE.LockOrder.shouldbe_later_lk->guestaddr);
          if (xe->XE.LockOrder.actual_earlier_ec) {
              VG_(pp_ExeContext)(xe->XE.LockOrder.actual_earlier_ec);
          } else {
@@ -1019,20 +996,23 @@ void HG_(pp_Error) ( Error* err )
          }
          emit( "\n" );
          emit(" followed by a later acquisition of lock at %p\n",
-              (void*)xe->XE.LockOrder.shouldbe_earlier_ga);
+              (void*)xe->XE.LockOrder.shouldbe_earlier_lk->guestaddr);
          VG_(pp_ExeContext)( VG_(get_error_where)(err) );
          if (xe->XE.LockOrder.shouldbe_earlier_ec
              && xe->XE.LockOrder.shouldbe_later_ec) {
             emit("\n");
             emit( "Required order was established by "
                   "acquisition of lock at %p\n",
-                  (void*)xe->XE.LockOrder.shouldbe_earlier_ga );
+                  (void*)xe->XE.LockOrder.shouldbe_earlier_lk->guestaddr );
             VG_(pp_ExeContext)( xe->XE.LockOrder.shouldbe_earlier_ec );
             emit( "\n" );
             emit( " followed by a later acquisition of lock at %p\n",
-                  (void*)xe->XE.LockOrder.shouldbe_later_ga );
+                  (void*)xe->XE.LockOrder.shouldbe_later_lk->guestaddr );
             VG_(pp_ExeContext)( xe->XE.LockOrder.shouldbe_later_ec );
          }
+         emit("\n");
+         announce_LockP ( xe->XE.LockOrder.shouldbe_earlier_lk );
+         announce_LockP ( xe->XE.LockOrder.shouldbe_later_lk );
 
       }
 
@@ -1116,12 +1096,7 @@ void HG_(pp_Error) ( Error* err )
                (Int)xe->XE.UnlockForeign.owner->errmsg_index );
          emit( "  </xwhat>\n" );
          VG_(pp_ExeContext)( VG_(get_error_where)(err) );
-
-         if (xe->XE.UnlockForeign.lock->appeared_at) {
-            emit( "  <auxwhat>Lock at %p was first observed</auxwhat>\n",
-                  (void*)xe->XE.UnlockForeign.lock->guestaddr );
-            VG_(pp_ExeContext)( xe->XE.UnlockForeign.lock->appeared_at );
-         }
+         announce_LockP ( xe->XE.UnlockForeign.lock );
 
       } else {
 
@@ -1131,11 +1106,7 @@ void HG_(pp_Error) ( Error* err )
                (void*)xe->XE.UnlockForeign.lock->guestaddr,
                (Int)xe->XE.UnlockForeign.owner->errmsg_index );
          VG_(pp_ExeContext)( VG_(get_error_where)(err) );
-         if (xe->XE.UnlockForeign.lock->appeared_at) {
-            emit( "  Lock at %p was first observed\n",
-                  (void*)xe->XE.UnlockForeign.lock->guestaddr );
-            VG_(pp_ExeContext)( xe->XE.UnlockForeign.lock->appeared_at );
-         }
+         announce_LockP ( xe->XE.UnlockForeign.lock );
 
       }
 
@@ -1157,11 +1128,7 @@ void HG_(pp_Error) ( Error* err )
                (Int)xe->XE.UnlockUnlocked.thr->errmsg_index );
          emit( "  </xwhat>\n" );
          VG_(pp_ExeContext)( VG_(get_error_where)(err) );
-         if (xe->XE.UnlockUnlocked.lock->appeared_at) {
-            emit( "  <auxwhat>Lock at %p was first observed</auxwhat>\n",
-                  (void*)xe->XE.UnlockUnlocked.lock->guestaddr );
-            VG_(pp_ExeContext)( xe->XE.UnlockUnlocked.lock->appeared_at );
-         }
+         announce_LockP ( xe->XE.UnlockUnlocked.lock);
 
       } else {
 
@@ -1169,11 +1136,7 @@ void HG_(pp_Error) ( Error* err )
                (Int)xe->XE.UnlockUnlocked.thr->errmsg_index,
                (void*)xe->XE.UnlockUnlocked.lock->guestaddr );
          VG_(pp_ExeContext)( VG_(get_error_where)(err) );
-         if (xe->XE.UnlockUnlocked.lock->appeared_at) {
-            emit( "  Lock at %p was first observed\n",
-                  (void*)xe->XE.UnlockUnlocked.lock->guestaddr );
-            VG_(pp_ExeContext)( xe->XE.UnlockUnlocked.lock->appeared_at );
-         }
+         announce_LockP ( xe->XE.UnlockUnlocked.lock);
 
       }
 
@@ -1182,7 +1145,7 @@ void HG_(pp_Error) ( Error* err )
 
    case XE_Race: {
       Addr      err_ga;
-      HChar*    what;
+      const HChar* what;
       Int       szB;
       what      = xe->XE.Race.isWrite ? "write" : "read";
       szB       = xe->XE.Race.szB;
@@ -1284,47 +1247,7 @@ void HG_(pp_Error) ( Error* err )
          }
 
       }
-
-      /* If we have a description of the address in terms of a heap
-         block, show it. */
-      if (xe->XE.Race.hctxt) {
-         SizeT delta = err_ga - xe->XE.Race.haddr;
-         if (xml) {
-            emit("  <auxwhat>Address %p is %ld bytes inside a block "
-                 "of size %ld alloc'd</auxwhat>\n", (void*)err_ga, delta, 
-                 xe->XE.Race.hszB);
-            VG_(pp_ExeContext)( xe->XE.Race.hctxt );
-         } else {
-            emit("\n");
-            emit("Address %p is %ld bytes inside a block "
-                 "of size %ld alloc'd\n", (void*)err_ga, delta, 
-                 xe->XE.Race.hszB);
-            VG_(pp_ExeContext)( xe->XE.Race.hctxt );
-         }
-      }
-
-      /* If we have a better description of the address, show it.
-         Note that in XML mode, it will already by nicely wrapped up
-         in tags, either <auxwhat> or <xauxwhat>, so we can just emit
-         it verbatim. */
-      if (xml) {
-         if (xe->XE.Race.descr1)
-            emit( "  %s\n",
-                  (HChar*)VG_(indexXA)( xe->XE.Race.descr1, 0 ) );
-         if (xe->XE.Race.descr2)
-            emit( "  %s\n",
-                  (HChar*)VG_(indexXA)( xe->XE.Race.descr2, 0 ) );
-      } else {
-         if (xe->XE.Race.descr1 || xe->XE.Race.descr2)
-            emit("\n");
-         if (xe->XE.Race.descr1)
-            emit( "%s\n",
-                  (HChar*)VG_(indexXA)( xe->XE.Race.descr1, 0 ) );
-         if (xe->XE.Race.descr2)
-            emit( "%s\n",
-                  (HChar*)VG_(indexXA)( xe->XE.Race.descr2, 0 ) );
-      }
-
+      VG_(pp_addrinfo) (err_ga, &xe->XE.Race.data_addrinfo);
       break; /* case XE_Race */
    } /* case XE_Race */
 
@@ -1333,7 +1256,7 @@ void HG_(pp_Error) ( Error* err )
    } /* switch (VG_(get_error_kind)(err)) */
 }
 
-Char* HG_(get_error_name) ( Error* err )
+const HChar* HG_(get_error_name) ( Error* err )
 {
    switch (VG_(get_error_kind)(err)) {
       case XE_Race:           return "Race";
@@ -1347,7 +1270,7 @@ Char* HG_(get_error_name) ( Error* err )
    }
 }
 
-Bool HG_(recognised_suppression) ( Char* name, Supp *su )
+Bool HG_(recognised_suppression) ( const HChar* name, Supp *su )
 {
 #  define TRY(_name,_xskind)                   \
       if (0 == VG_(strcmp)(name, (_name))) {   \
@@ -1366,8 +1289,8 @@ Bool HG_(recognised_suppression) ( Char* name, Supp *su )
 #  undef TRY
 }
 
-Bool HG_(read_extra_suppression_info) ( Int fd, Char** bufpp, SizeT* nBufp,
-                                        Supp* su )
+Bool HG_(read_extra_suppression_info) ( Int fd, HChar** bufpp, SizeT* nBufp,
+                                        Int* lineno, Supp* su )
 {
    /* do nothing -- no extra suppression info present.  Return True to
       indicate nothing bad happened. */
@@ -1390,12 +1313,25 @@ Bool HG_(error_matches_suppression) ( Error* err, Supp* su )
 }
 
 Bool HG_(get_extra_suppression_info) ( Error* err,
-                                       /*OUT*/Char* buf, Int nBuf )
+                                       /*OUT*/HChar* buf, Int nBuf )
 {
    /* Do nothing */
    return False;
 }
 
+Bool HG_(print_extra_suppression_use) ( Supp* su,
+                                        /*OUT*/HChar* buf, Int nBuf )
+{
+   /* Do nothing */
+   return False;
+}
+
+void HG_(update_extra_suppression_use) ( Error* err, Supp* su )
+{
+   /* Do nothing */
+   return;
+}
+
 
 /*--------------------------------------------------------------------*/
 /*--- end                                              hg_errors.c ---*/
