@@ -348,6 +348,10 @@ struct vki_sigcontext {
 #define VKI_F_SETOWN_EX     15
 #define VKI_F_GETOWN_EX     16
 
+#define VKI_F_OFD_GETLK     36
+#define VKI_F_OFD_SETLK     37
+#define VKI_F_OFD_SETLKW    38
+
 #define VKI_F_GETLK64       33          /*  using 'struct flock64' */
 #define VKI_F_SETLK64       34
 #define VKI_F_SETLKW64      35
@@ -387,6 +391,7 @@ struct vki_f_owner_ex {
 
 #define VKI_SIOCSPGRP           0x8902
 #define VKI_SIOCGPGRP           0x8904
+#define VKI_SIOCATMARK          0x8905
 #define VKI_SIOCGSTAMP          0x8906      /* Get stamp (timeval) */
 #define VKI_SIOCGSTAMPNS        0x8907      /* Get stamp (timespec) */
 
@@ -530,8 +535,8 @@ struct vki_termios {
 
 #define _VKI_IOC_NRBITS     8
 #define _VKI_IOC_TYPEBITS   8
-#define _VKI_IOC_SIZEBITS   14
-#define _VKI_IOC_DIRBITS    2
+#define _VKI_IOC_SIZEBITS   13
+#define _VKI_IOC_DIRBITS    3
 
 #define _VKI_IOC_NRMASK     ((1 << _VKI_IOC_NRBITS)-1)
 #define _VKI_IOC_TYPEMASK   ((1 << _VKI_IOC_TYPEBITS)-1)
@@ -544,8 +549,8 @@ struct vki_termios {
 #define _VKI_IOC_DIRSHIFT   (_VKI_IOC_SIZESHIFT+_VKI_IOC_SIZEBITS)
 
 #define _VKI_IOC_NONE   1U
-#define _VKI_IOC_WRITE  2U
-#define _VKI_IOC_READ   4U
+#define _VKI_IOC_READ   2U
+#define _VKI_IOC_WRITE  4U
 
 #define _VKI_IOC(dir,type,nr,size) \
                 (((dir)  << _VKI_IOC_DIRSHIFT) | \
@@ -999,6 +1004,13 @@ enum vki_sock_type {
 };
 #define ARCH_HAS_SOCKET_TYPES 1
 
+//----------------------------------------------------------------------
+// From linux-3.13.0/include/asm/errno.h
+//----------------------------------------------------------------------
+
+#define	VKI_ENOSYS       89  /* Function not implemented */
+#define	VKI_EOVERFLOW    79  /* Value too large for defined data type */
+
 #endif // __VKI_MIPS64_LINUX_H
 
 /*--------------------------------------------------------------------*/
